# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AuditSSHSettingUpdateParams"]


class AuditSSHSettingUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    public_key: Required[str]
    """Provide the Base64-encoded HPKE public key that encrypts SSH session logs.

    See
    https://developers.cloudflare.com/cloudflare-one/connections/connect-networks/use-cases/ssh/ssh-infrastructure-access/#enable-ssh-command-logging.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ValidateDestinationParams"]


class ValidateDestinationParams(TypedDict, total=False):
    destination_conf: Required[str]
    """Uniquely identifies a resource (such as an s3 bucket) where data.

    will be pushed. Additional configuration parameters supported by the destination
    may be included.
    """

    account_id: str
    """The Account ID to use for this endpoint. Mutually exclusive with the Zone ID."""

    zone_id: str
    """The Zone ID to use for this endpoint. Mutually exclusive with the Account ID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from ...._types import SequenceNotStr

__all__ = ["DHCPRelayParam"]


class DHCPRelayParam(TypedDict, total=False):
    server_addresses: SequenceNotStr[str]
    """List of DHCP server IPs."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OverrideListParams"]


class OverrideListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    page: float
    """The page number of paginated results."""

    per_page: float
    """The number of WAF overrides per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tunnels import (
    TunnelsResource,
    AsyncTunnelsResource,
    TunnelsResourceWithRawResponse,
    AsyncTunnelsResourceWithRawResponse,
    TunnelsResourceWithStreamingResponse,
    AsyncTunnelsResourceWithStreamingResponse,
)
from .cloudflared import (
    CloudflaredResource,
    AsyncCloudflaredResource,
    CloudflaredResourceWithRawResponse,
    AsyncCloudflaredResourceWithRawResponse,
    CloudflaredResourceWithStreamingResponse,
    AsyncCloudflaredResourceWithStreamingResponse,
)
from .warp_connector import (
    WARPConnectorResource,
    AsyncWARPConnectorResource,
    WARPConnectorResourceWithRawResponse,
    AsyncWARPConnectorResourceWithRawResponse,
    WARPConnectorResourceWithStreamingResponse,
    AsyncWARPConnectorResourceWithStreamingResponse,
)

__all__ = [
    "CloudflaredResource",
    "AsyncCloudflaredResource",
    "CloudflaredResourceWithRawResponse",
    "AsyncCloudflaredResourceWithRawResponse",
    "CloudflaredResourceWithStreamingResponse",
    "AsyncCloudflaredResourceWithStreamingResponse",
    "WARPConnectorResource",
    "AsyncWARPConnectorResource",
    "WARPConnectorResourceWithRawResponse",
    "AsyncWARPConnectorResourceWithRawResponse",
    "WARPConnectorResourceWithStreamingResponse",
    "AsyncWARPConnectorResourceWithStreamingResponse",
    "TunnelsResource",
    "AsyncTunnelsResource",
    "TunnelsResourceWithRawResponse",
    "AsyncTunnelsResourceWithRawResponse",
    "TunnelsResourceWithStreamingResponse",
    "AsyncTunnelsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import path_template
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from .language.language import (
    LanguageResource,
    AsyncLanguageResource,
    LanguageResourceWithRawResponse,
    AsyncLanguageResourceWithRawResponse,
    LanguageResourceWithStreamingResponse,
    AsyncLanguageResourceWithStreamingResponse,
)
from ....types.stream.caption import Caption

__all__ = ["CaptionsResource", "AsyncCaptionsResource"]


class CaptionsResource(SyncAPIResource):
    @cached_property
    def language(self) -> LanguageResource:
        return LanguageResource(self._client)

    @cached_property
    def with_raw_response(self) -> CaptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CaptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CaptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CaptionsResourceWithStreamingResponse(self)

    def get(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[Caption]:
        """
        Lists the available captions or subtitles for a specific video.

        Args:
          account_id: Identifier.

          identifier: A Cloudflare-generated unique identifier for a media item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/stream/{identifier}/captions", account_id=account_id, identifier=identifier
            ),
            page=SyncSinglePage[Caption],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Caption,
        )


class AsyncCaptionsResource(AsyncAPIResource):
    @cached_property
    def language(self) -> AsyncLanguageResource:
        return AsyncLanguageResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCaptionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCaptionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCaptionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCaptionsResourceWithStreamingResponse(self)

    def get(
        self,
        identifier: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Caption, AsyncSinglePage[Caption]]:
        """
        Lists the available captions or subtitles for a specific video.

        Args:
          account_id: Identifier.

          identifier: A Cloudflare-generated unique identifier for a media item.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not identifier:
            raise ValueError(f"Expected a non-empty value for `identifier` but received {identifier!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/stream/{identifier}/captions", account_id=account_id, identifier=identifier
            ),
            page=AsyncSinglePage[Caption],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=Caption,
        )


class CaptionsResourceWithRawResponse:
    def __init__(self, captions: CaptionsResource) -> None:
        self._captions = captions

        self.get = to_raw_response_wrapper(
            captions.get,
        )

    @cached_property
    def language(self) -> LanguageResourceWithRawResponse:
        return LanguageResourceWithRawResponse(self._captions.language)


class AsyncCaptionsResourceWithRawResponse:
    def __init__(self, captions: AsyncCaptionsResource) -> None:
        self._captions = captions

        self.get = async_to_raw_response_wrapper(
            captions.get,
        )

    @cached_property
    def language(self) -> AsyncLanguageResourceWithRawResponse:
        return AsyncLanguageResourceWithRawResponse(self._captions.language)


class CaptionsResourceWithStreamingResponse:
    def __init__(self, captions: CaptionsResource) -> None:
        self._captions = captions

        self.get = to_streamed_response_wrapper(
            captions.get,
        )

    @cached_property
    def language(self) -> LanguageResourceWithStreamingResponse:
        return LanguageResourceWithStreamingResponse(self._captions.language)


class AsyncCaptionsResourceWithStreamingResponse:
    def __init__(self, captions: AsyncCaptionsResource) -> None:
        self._captions = captions

        self.get = async_to_streamed_response_wrapper(
            captions.get,
        )

    @cached_property
    def language(self) -> AsyncLanguageResourceWithStreamingResponse:
        return AsyncLanguageResourceWithStreamingResponse(self._captions.language)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.zones.available_rate_plan import AvailableRatePlan

__all__ = ["PlansResource", "AsyncPlansResource"]


class PlansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[AvailableRatePlan]:
        """
        Lists available plans the zone can subscribe to.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/available_plans", zone_id=zone_id),
            page=SyncSinglePage[AvailableRatePlan],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AvailableRatePlan,
        )

    def get(
        self,
        plan_identifier: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AvailableRatePlan:
        """
        Details of the available plan that the zone can subscribe to.

        Args:
          zone_id: Identifier

          plan_identifier: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not plan_identifier:
            raise ValueError(f"Expected a non-empty value for `plan_identifier` but received {plan_identifier!r}")
        return self._get(
            path_template(
                "/zones/{zone_id}/available_plans/{plan_identifier}", zone_id=zone_id, plan_identifier=plan_identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AvailableRatePlan]._unwrapper,
            ),
            cast_to=cast(Type[AvailableRatePlan], ResultWrapper[AvailableRatePlan]),
        )


class AsyncPlansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPlansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPlansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPlansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPlansResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AvailableRatePlan, AsyncSinglePage[AvailableRatePlan]]:
        """
        Lists available plans the zone can subscribe to.

        Args:
          zone_id: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/available_plans", zone_id=zone_id),
            page=AsyncSinglePage[AvailableRatePlan],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=AvailableRatePlan,
        )

    async def get(
        self,
        plan_identifier: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AvailableRatePlan:
        """
        Details of the available plan that the zone can subscribe to.

        Args:
          zone_id: Identifier

          plan_identifier: Identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not plan_identifier:
            raise ValueError(f"Expected a non-empty value for `plan_identifier` but received {plan_identifier!r}")
        return await self._get(
            path_template(
                "/zones/{zone_id}/available_plans/{plan_identifier}", zone_id=zone_id, plan_identifier=plan_identifier
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AvailableRatePlan]._unwrapper,
            ),
            cast_to=cast(Type[AvailableRatePlan], ResultWrapper[AvailableRatePlan]),
        )


class PlansResourceWithRawResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_raw_response_wrapper(
            plans.list,
        )
        self.get = to_raw_response_wrapper(
            plans.get,
        )


class AsyncPlansResourceWithRawResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_raw_response_wrapper(
            plans.list,
        )
        self.get = async_to_raw_response_wrapper(
            plans.get,
        )


class PlansResourceWithStreamingResponse:
    def __init__(self, plans: PlansResource) -> None:
        self._plans = plans

        self.list = to_streamed_response_wrapper(
            plans.list,
        )
        self.get = to_streamed_response_wrapper(
            plans.get,
        )


class AsyncPlansResourceWithStreamingResponse:
    def __init__(self, plans: AsyncPlansResource) -> None:
        self._plans = plans

        self.list = async_to_streamed_response_wrapper(
            plans.list,
        )
        self.get = async_to_streamed_response_wrapper(
            plans.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .pages import (
    PagesResource,
    AsyncPagesResource,
    PagesResourceWithRawResponse,
    AsyncPagesResourceWithRawResponse,
    PagesResourceWithStreamingResponse,
    AsyncPagesResourceWithStreamingResponse,
)
from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)

__all__ = [
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "PagesResource",
    "AsyncPagesResource",
    "PagesResourceWithRawResponse",
    "AsyncPagesResourceWithRawResponse",
    "PagesResourceWithStreamingResponse",
    "AsyncPagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .ttl_param import TTLParam
from .record_tags import RecordTags
from .a_record_param import ARecordParam
from .ds_record_param import DSRecordParam
from .mx_record_param import MXRecordParam
from .ns_record_param import NSRecordParam
from .caa_record_param import CAARecordParam
from .loc_record_param import LOCRecordParam
from .ptr_record_param import PTRRecordParam
from .srv_record_param import SRVRecordParam
from .txt_record_param import TXTRecordParam
from .uri_record_param import URIRecordParam
from .aaaa_record_param import AAAARecordParam
from .cert_record_param import CERTRecordParam
from .svcb_record_param import SVCBRecordParam
from .tlsa_record_param import TLSARecordParam
from .cname_record_param import CNAMERecordParam
from .https_record_param import HTTPSRecordParam
from .naptr_record_param import NAPTRRecordParam
from .sshfp_record_param import SSHFPRecordParam
from .dnskey_record_param import DNSKEYRecordParam
from .smimea_record_param import SMIMEARecordParam

__all__ = [
    "BatchPutParam",
    "ARecord",
    "AAAARecord",
    "CNAMERecord",
    "MXRecord",
    "NSRecord",
    "OpenpgpkeyRecord",
    "OpenpgpkeyRecordSettings",
    "PTRRecord",
    "TXTRecord",
    "CAARecord",
    "CERTRecord",
    "DNSKEYRecord",
    "DSRecord",
    "HTTPSRecord",
    "LOCRecord",
    "NAPTRRecord",
    "SMIMEARecord",
    "SRVRecord",
    "SSHFPRecord",
    "SVCBRecord",
    "TLSARecord",
    "URIRecord",
]


class ARecord(ARecordParam, total=False):
    id: Required[str]
    """Identifier."""


class AAAARecord(AAAARecordParam, total=False):
    id: Required[str]
    """Identifier."""


class CNAMERecord(CNAMERecordParam, total=False):
    id: Required[str]
    """Identifier."""


class MXRecord(MXRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class NSRecord(NSRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class OpenpgpkeyRecordSettings(TypedDict, total=False):
    """Settings for the DNS record."""

    ipv4_only: bool
    """
    When enabled, only A records will be generated, and AAAA records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """

    ipv6_only: bool
    """
    When enabled, only AAAA records will be generated, and A records will not be
    created. This setting is intended for exceptional cases. Note that this option
    only applies to proxied records and it has no effect on whether Cloudflare
    communicates with the origin using IPv4 or IPv6.
    """


class OpenpgpkeyRecord(TypedDict, total=False):
    id: Required[str]
    """Identifier."""

    name: Required[str]
    """Complete DNS record name, including the zone name, in Punycode."""

    ttl: Required[TTLParam]
    """Time To Live (TTL) of the DNS record in seconds.

    Setting to 1 means 'automatic'. Value must be between 60 and 86400, with the
    minimum reduced to 30 for Enterprise zones.
    """

    type: Required[Literal["OPENPGPKEY"]]
    """Record type."""

    comment: str
    """Comments or notes about the DNS record.

    This field has no effect on DNS responses.
    """

    content: str
    """A single Base64-encoded OpenPGP Transferable Public Key (RFC 4880 Section 11.1)"""

    proxied: bool
    """
    Whether the record is receiving the performance and security benefits of
    Cloudflare.
    """

    settings: OpenpgpkeyRecordSettings
    """Settings for the DNS record."""

    tags: SequenceNotStr[RecordTags]
    """Custom tags for the DNS record. This field has no effect on DNS responses."""


class PTRRecord(PTRRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class TXTRecord(TXTRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class CAARecord(CAARecordParam, total=False):
    id: Required[str]
    """Identifier."""


class CERTRecord(CERTRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class DNSKEYRecord(DNSKEYRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class DSRecord(DSRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class HTTPSRecord(HTTPSRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class LOCRecord(LOCRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class NAPTRRecord(NAPTRRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class SMIMEARecord(SMIMEARecordParam, total=False):
    id: Required[str]
    """Identifier."""


class SRVRecord(SRVRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class SSHFPRecord(SSHFPRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class SVCBRecord(SVCBRecordParam, total=False):
    id: Required[str]
    """Identifier."""


class TLSARecord(TLSARecordParam, total=False):
    id: Required[str]
    """Identifier."""


class URIRecord(URIRecordParam, total=False):
    id: Required[str]
    """Identifier."""


BatchPutParam: TypeAlias = Union[
    ARecord,
    AAAARecord,
    CNAMERecord,
    MXRecord,
    NSRecord,
    OpenpgpkeyRecord,
    PTRRecord,
    TXTRecord,
    CAARecord,
    CERTRecord,
    DNSKEYRecord,
    DSRecord,
    HTTPSRecord,
    LOCRecord,
    NAPTRRecord,
    SMIMEARecord,
    SRVRecord,
    SSHFPRecord,
    SVCBRecord,
    TLSARecord,
    URIRecord,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ...._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.magic_cloud_networking.on_ramps import address_space_edit_params, address_space_update_params
from ....types.magic_cloud_networking.on_ramps.address_space_edit_response import AddressSpaceEditResponse
from ....types.magic_cloud_networking.on_ramps.address_space_list_response import AddressSpaceListResponse
from ....types.magic_cloud_networking.on_ramps.address_space_update_response import AddressSpaceUpdateResponse

__all__ = ["AddressSpacesResource", "AsyncAddressSpacesResource"]


class AddressSpacesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AddressSpacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AddressSpacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AddressSpacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AddressSpacesResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        account_id: str,
        prefixes: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceUpdateResponse:
        """
        Update the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._put(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            body=maybe_transform({"prefixes": prefixes}, address_space_update_params.AddressSpaceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceUpdateResponse], ResultWrapper[AddressSpaceUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceListResponse:
        """
        Read the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceListResponse], ResultWrapper[AddressSpaceListResponse]),
        )

    def edit(
        self,
        *,
        account_id: str,
        prefixes: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceEditResponse:
        """
        Update the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._patch(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            body=maybe_transform({"prefixes": prefixes}, address_space_edit_params.AddressSpaceEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceEditResponse], ResultWrapper[AddressSpaceEditResponse]),
        )


class AsyncAddressSpacesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAddressSpacesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAddressSpacesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAddressSpacesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAddressSpacesResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        account_id: str,
        prefixes: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceUpdateResponse:
        """
        Update the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._put(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            body=await async_maybe_transform(
                {"prefixes": prefixes}, address_space_update_params.AddressSpaceUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceUpdateResponse], ResultWrapper[AddressSpaceUpdateResponse]),
        )

    async def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceListResponse:
        """
        Read the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceListResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceListResponse], ResultWrapper[AddressSpaceListResponse]),
        )

    async def edit(
        self,
        *,
        account_id: str,
        prefixes: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddressSpaceEditResponse:
        """
        Update the Magic WAN Address Space (Closed Beta).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._patch(
            path_template("/accounts/{account_id}/magic/cloud/onramps/magic_wan_address_space", account_id=account_id),
            body=await async_maybe_transform({"prefixes": prefixes}, address_space_edit_params.AddressSpaceEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AddressSpaceEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AddressSpaceEditResponse], ResultWrapper[AddressSpaceEditResponse]),
        )


class AddressSpacesResourceWithRawResponse:
    def __init__(self, address_spaces: AddressSpacesResource) -> None:
        self._address_spaces = address_spaces

        self.update = to_raw_response_wrapper(
            address_spaces.update,
        )
        self.list = to_raw_response_wrapper(
            address_spaces.list,
        )
        self.edit = to_raw_response_wrapper(
            address_spaces.edit,
        )


class AsyncAddressSpacesResourceWithRawResponse:
    def __init__(self, address_spaces: AsyncAddressSpacesResource) -> None:
        self._address_spaces = address_spaces

        self.update = async_to_raw_response_wrapper(
            address_spaces.update,
        )
        self.list = async_to_raw_response_wrapper(
            address_spaces.list,
        )
        self.edit = async_to_raw_response_wrapper(
            address_spaces.edit,
        )


class AddressSpacesResourceWithStreamingResponse:
    def __init__(self, address_spaces: AddressSpacesResource) -> None:
        self._address_spaces = address_spaces

        self.update = to_streamed_response_wrapper(
            address_spaces.update,
        )
        self.list = to_streamed_response_wrapper(
            address_spaces.list,
        )
        self.edit = to_streamed_response_wrapper(
            address_spaces.edit,
        )


class AsyncAddressSpacesResourceWithStreamingResponse:
    def __init__(self, address_spaces: AsyncAddressSpacesResource) -> None:
        self._address_spaces = address_spaces

        self.update = async_to_streamed_response_wrapper(
            address_spaces.update,
        )
        self.list = async_to_streamed_response_wrapper(
            address_spaces.list,
        )
        self.edit = async_to_streamed_response_wrapper(
            address_spaces.edit,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .hostname_associations import (
    HostnameAssociationsResource,
    AsyncHostnameAssociationsResource,
    HostnameAssociationsResourceWithRawResponse,
    AsyncHostnameAssociationsResourceWithRawResponse,
    HostnameAssociationsResourceWithStreamingResponse,
    AsyncHostnameAssociationsResourceWithStreamingResponse,
)

__all__ = ["CertificateAuthoritiesResource", "AsyncCertificateAuthoritiesResource"]


class CertificateAuthoritiesResource(SyncAPIResource):
    @cached_property
    def hostname_associations(self) -> HostnameAssociationsResource:
        return HostnameAssociationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CertificateAuthoritiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CertificateAuthoritiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CertificateAuthoritiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CertificateAuthoritiesResourceWithStreamingResponse(self)


class AsyncCertificateAuthoritiesResource(AsyncAPIResource):
    @cached_property
    def hostname_associations(self) -> AsyncHostnameAssociationsResource:
        return AsyncHostnameAssociationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCertificateAuthoritiesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCertificateAuthoritiesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCertificateAuthoritiesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCertificateAuthoritiesResourceWithStreamingResponse(self)


class CertificateAuthoritiesResourceWithRawResponse:
    def __init__(self, certificate_authorities: CertificateAuthoritiesResource) -> None:
        self._certificate_authorities = certificate_authorities

    @cached_property
    def hostname_associations(self) -> HostnameAssociationsResourceWithRawResponse:
        return HostnameAssociationsResourceWithRawResponse(self._certificate_authorities.hostname_associations)


class AsyncCertificateAuthoritiesResourceWithRawResponse:
    def __init__(self, certificate_authorities: AsyncCertificateAuthoritiesResource) -> None:
        self._certificate_authorities = certificate_authorities

    @cached_property
    def hostname_associations(self) -> AsyncHostnameAssociationsResourceWithRawResponse:
        return AsyncHostnameAssociationsResourceWithRawResponse(self._certificate_authorities.hostname_associations)


class CertificateAuthoritiesResourceWithStreamingResponse:
    def __init__(self, certificate_authorities: CertificateAuthoritiesResource) -> None:
        self._certificate_authorities = certificate_authorities

    @cached_property
    def hostname_associations(self) -> HostnameAssociationsResourceWithStreamingResponse:
        return HostnameAssociationsResourceWithStreamingResponse(self._certificate_authorities.hostname_associations)


class AsyncCertificateAuthoritiesResourceWithStreamingResponse:
    def __init__(self, certificate_authorities: AsyncCertificateAuthoritiesResource) -> None:
        self._certificate_authorities = certificate_authorities

    @cached_property
    def hostname_associations(self) -> AsyncHostnameAssociationsResourceWithStreamingResponse:
        return AsyncHostnameAssociationsResourceWithStreamingResponse(
            self._certificate_authorities.hostname_associations
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .summary import (
    SummaryResource,
    AsyncSummaryResource,
    SummaryResourceWithRawResponse,
    AsyncSummaryResourceWithRawResponse,
    SummaryResourceWithStreamingResponse,
    AsyncSummaryResourceWithStreamingResponse,
)
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource

__all__ = ["TimeseriesGroupsResource", "AsyncTimeseriesGroupsResource"]


class TimeseriesGroupsResource(SyncAPIResource):
    @cached_property
    def summary(self) -> SummaryResource:
        return SummaryResource(self._client)

    @cached_property
    def with_raw_response(self) -> TimeseriesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return TimeseriesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TimeseriesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return TimeseriesGroupsResourceWithStreamingResponse(self)


class AsyncTimeseriesGroupsResource(AsyncAPIResource):
    @cached_property
    def summary(self) -> AsyncSummaryResource:
        return AsyncSummaryResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTimeseriesGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTimeseriesGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTimeseriesGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncTimeseriesGroupsResourceWithStreamingResponse(self)


class TimeseriesGroupsResourceWithRawResponse:
    def __init__(self, timeseries_groups: TimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

    @cached_property
    def summary(self) -> SummaryResourceWithRawResponse:
        return SummaryResourceWithRawResponse(self._timeseries_groups.summary)


class AsyncTimeseriesGroupsResourceWithRawResponse:
    def __init__(self, timeseries_groups: AsyncTimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithRawResponse:
        return AsyncSummaryResourceWithRawResponse(self._timeseries_groups.summary)


class TimeseriesGroupsResourceWithStreamingResponse:
    def __init__(self, timeseries_groups: TimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

    @cached_property
    def summary(self) -> SummaryResourceWithStreamingResponse:
        return SummaryResourceWithStreamingResponse(self._timeseries_groups.summary)


class AsyncTimeseriesGroupsResourceWithStreamingResponse:
    def __init__(self, timeseries_groups: AsyncTimeseriesGroupsResource) -> None:
        self._timeseries_groups = timeseries_groups

    @cached_property
    def summary(self) -> AsyncSummaryResourceWithStreamingResponse:
        return AsyncSummaryResourceWithStreamingResponse(self._timeseries_groups.summary)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CountryRuleParam", "Geo"]


class Geo(TypedDict, total=False):
    country_code: Required[str]
    """The country code that should be matched."""


class CountryRuleParam(TypedDict, total=False):
    """Matches a specific country"""

    geo: Required[Geo]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .submissions import (
    SubmissionsResource,
    AsyncSubmissionsResource,
    SubmissionsResourceWithRawResponse,
    AsyncSubmissionsResourceWithRawResponse,
    SubmissionsResourceWithStreamingResponse,
    AsyncSubmissionsResourceWithStreamingResponse,
)
from .settings.settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .phishguard.phishguard import (
    PhishguardResource,
    AsyncPhishguardResource,
    PhishguardResourceWithRawResponse,
    AsyncPhishguardResourceWithRawResponse,
    PhishguardResourceWithStreamingResponse,
    AsyncPhishguardResourceWithStreamingResponse,
)
from .investigate.investigate import (
    InvestigateResource,
    AsyncInvestigateResource,
    InvestigateResourceWithRawResponse,
    AsyncInvestigateResourceWithRawResponse,
    InvestigateResourceWithStreamingResponse,
    AsyncInvestigateResourceWithStreamingResponse,
)

__all__ = ["EmailSecurityResource", "AsyncEmailSecurityResource"]


class EmailSecurityResource(SyncAPIResource):
    @cached_property
    def investigate(self) -> InvestigateResource:
        return InvestigateResource(self._client)

    @cached_property
    def phishguard(self) -> PhishguardResource:
        return PhishguardResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def submissions(self) -> SubmissionsResource:
        return SubmissionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EmailSecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EmailSecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmailSecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EmailSecurityResourceWithStreamingResponse(self)


class AsyncEmailSecurityResource(AsyncAPIResource):
    @cached_property
    def investigate(self) -> AsyncInvestigateResource:
        return AsyncInvestigateResource(self._client)

    @cached_property
    def phishguard(self) -> AsyncPhishguardResource:
        return AsyncPhishguardResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def submissions(self) -> AsyncSubmissionsResource:
        return AsyncSubmissionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEmailSecurityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmailSecurityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmailSecurityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEmailSecurityResourceWithStreamingResponse(self)


class EmailSecurityResourceWithRawResponse:
    def __init__(self, email_security: EmailSecurityResource) -> None:
        self._email_security = email_security

    @cached_property
    def investigate(self) -> InvestigateResourceWithRawResponse:
        return InvestigateResourceWithRawResponse(self._email_security.investigate)

    @cached_property
    def phishguard(self) -> PhishguardResourceWithRawResponse:
        return PhishguardResourceWithRawResponse(self._email_security.phishguard)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._email_security.settings)

    @cached_property
    def submissions(self) -> SubmissionsResourceWithRawResponse:
        return SubmissionsResourceWithRawResponse(self._email_security.submissions)


class AsyncEmailSecurityResourceWithRawResponse:
    def __init__(self, email_security: AsyncEmailSecurityResource) -> None:
        self._email_security = email_security

    @cached_property
    def investigate(self) -> AsyncInvestigateResourceWithRawResponse:
        return AsyncInvestigateResourceWithRawResponse(self._email_security.investigate)

    @cached_property
    def phishguard(self) -> AsyncPhishguardResourceWithRawResponse:
        return AsyncPhishguardResourceWithRawResponse(self._email_security.phishguard)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._email_security.settings)

    @cached_property
    def submissions(self) -> AsyncSubmissionsResourceWithRawResponse:
        return AsyncSubmissionsResourceWithRawResponse(self._email_security.submissions)


class EmailSecurityResourceWithStreamingResponse:
    def __init__(self, email_security: EmailSecurityResource) -> None:
        self._email_security = email_security

    @cached_property
    def investigate(self) -> InvestigateResourceWithStreamingResponse:
        return InvestigateResourceWithStreamingResponse(self._email_security.investigate)

    @cached_property
    def phishguard(self) -> PhishguardResourceWithStreamingResponse:
        return PhishguardResourceWithStreamingResponse(self._email_security.phishguard)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._email_security.settings)

    @cached_property
    def submissions(self) -> SubmissionsResourceWithStreamingResponse:
        return SubmissionsResourceWithStreamingResponse(self._email_security.submissions)


class AsyncEmailSecurityResourceWithStreamingResponse:
    def __init__(self, email_security: AsyncEmailSecurityResource) -> None:
        self._email_security = email_security

    @cached_property
    def investigate(self) -> AsyncInvestigateResourceWithStreamingResponse:
        return AsyncInvestigateResourceWithStreamingResponse(self._email_security.investigate)

    @cached_property
    def phishguard(self) -> AsyncPhishguardResourceWithStreamingResponse:
        return AsyncPhishguardResourceWithStreamingResponse(self._email_security.phishguard)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._email_security.settings)

    @cached_property
    def submissions(self) -> AsyncSubmissionsResourceWithStreamingResponse:
        return AsyncSubmissionsResourceWithStreamingResponse(self._email_security.submissions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.ai_gateway import evaluation_list_params, evaluation_create_params
from ...types.ai_gateway.evaluation_get_response import EvaluationGetResponse
from ...types.ai_gateway.evaluation_list_response import EvaluationListResponse
from ...types.ai_gateway.evaluation_create_response import EvaluationCreateResponse
from ...types.ai_gateway.evaluation_delete_response import EvaluationDeleteResponse

__all__ = ["EvaluationsResource", "AsyncEvaluationsResource"]


class EvaluationsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EvaluationsResourceWithStreamingResponse(self)

    def create(
        self,
        gateway_id: str,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str],
        evaluation_type_ids: SequenceNotStr[str],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            body=maybe_transform(
                {
                    "dataset_ids": dataset_ids,
                    "evaluation_type_ids": evaluation_type_ids,
                    "name": name,
                },
                evaluation_create_params.EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationCreateResponse], ResultWrapper[EvaluationCreateResponse]),
        )

    def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        processed: bool | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[EvaluationListResponse]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          gateway_id: gateway id

          search: Search by id, name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            page=SyncV4PagePaginationArray[EvaluationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "processed": processed,
                        "search": search,
                    },
                    evaluation_list_params.EvaluationListParams,
                ),
            ),
            model=EvaluationListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationDeleteResponse], ResultWrapper[EvaluationDeleteResponse]),
        )

    def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationGetResponse], ResultWrapper[EvaluationGetResponse]),
        )


class AsyncEvaluationsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEvaluationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEvaluationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEvaluationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEvaluationsResourceWithStreamingResponse(self)

    async def create(
        self,
        gateway_id: str,
        *,
        account_id: str,
        dataset_ids: SequenceNotStr[str],
        evaluation_type_ids: SequenceNotStr[str],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationCreateResponse:
        """
        Creates a new AI Gateway.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            body=await async_maybe_transform(
                {
                    "dataset_ids": dataset_ids,
                    "evaluation_type_ids": evaluation_type_ids,
                    "name": name,
                },
                evaluation_create_params.EvaluationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationCreateResponse], ResultWrapper[EvaluationCreateResponse]),
        )

    def list(
        self,
        gateway_id: str,
        *,
        account_id: str,
        name: str | Omit = omit,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        processed: bool | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[EvaluationListResponse, AsyncV4PagePaginationArray[EvaluationListResponse]]:
        """
        Lists all AI Gateway evaluator types configured for the account.

        Args:
          gateway_id: gateway id

          search: Search by id, name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations",
                account_id=account_id,
                gateway_id=gateway_id,
            ),
            page=AsyncV4PagePaginationArray[EvaluationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "name": name,
                        "page": page,
                        "per_page": per_page,
                        "processed": processed,
                        "search": search,
                    },
                    evaluation_list_params.EvaluationListParams,
                ),
            ),
            model=EvaluationListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationDeleteResponse:
        """
        Deletes an AI Gateway dataset.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationDeleteResponse], ResultWrapper[EvaluationDeleteResponse]),
        )

    async def get(
        self,
        id: str,
        *,
        account_id: str,
        gateway_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EvaluationGetResponse:
        """
        Retrieves details for a specific AI Gateway dataset.

        Args:
          gateway_id: gateway id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not gateway_id:
            raise ValueError(f"Expected a non-empty value for `gateway_id` but received {gateway_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/ai-gateway/gateways/{gateway_id}/evaluations/{id}",
                account_id=account_id,
                gateway_id=gateway_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EvaluationGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[EvaluationGetResponse], ResultWrapper[EvaluationGetResponse]),
        )


class EvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_raw_response_wrapper(
            evaluations.create,
        )
        self.list = to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = to_raw_response_wrapper(
            evaluations.delete,
        )
        self.get = to_raw_response_wrapper(
            evaluations.get,
        )


class AsyncEvaluationsResourceWithRawResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_raw_response_wrapper(
            evaluations.create,
        )
        self.list = async_to_raw_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_raw_response_wrapper(
            evaluations.delete,
        )
        self.get = async_to_raw_response_wrapper(
            evaluations.get,
        )


class EvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: EvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = to_streamed_response_wrapper(
            evaluations.create,
        )
        self.list = to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.get = to_streamed_response_wrapper(
            evaluations.get,
        )


class AsyncEvaluationsResourceWithStreamingResponse:
    def __init__(self, evaluations: AsyncEvaluationsResource) -> None:
        self._evaluations = evaluations

        self.create = async_to_streamed_response_wrapper(
            evaluations.create,
        )
        self.list = async_to_streamed_response_wrapper(
            evaluations.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            evaluations.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            evaluations.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from ...._models import BaseModel

__all__ = ["HealthCreateResponse"]


class HealthCreateResponse(BaseModel):
    pools: Optional[Dict[str, str]] = None
    """Monitored pool IDs mapped to their respective names."""

    preview_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["Delegations"]


class Delegations(BaseModel):
    id: Optional[str] = None
    """Identifier of a Delegation."""

    cidr: Optional[str] = None
    """IP Prefix in Classless Inter-Domain Routing format."""

    created_at: Optional[datetime] = None

    delegated_account_id: Optional[str] = None
    """Account identifier for the account to which prefix is being delegated."""

    modified_at: Optional[datetime] = None

    parent_prefix_id: Optional[str] = None
    """Identifier of an IP Prefix."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["BrowserCreateResponse"]


class BrowserCreateResponse(BaseModel):
    session_id: str = FieldInfo(alias="sessionId")
    """Browser session ID."""

    web_socket_debugger_url: Optional[str] = FieldInfo(alias="webSocketDebuggerUrl", default=None)
    """WebSocket URL for the session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.zero_trust.gateway import proxy_endpoint_edit_params, proxy_endpoint_create_params
from ....types.zero_trust.gateway.gateway_ips import GatewayIPs
from ....types.zero_trust.gateway.proxy_endpoint import ProxyEndpoint

__all__ = ["ProxyEndpointsResource", "AsyncProxyEndpointsResource"]


class ProxyEndpointsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProxyEndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ProxyEndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProxyEndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ProxyEndpointsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        name: str,
        kind: Literal["ip"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Create a new Zero Trust Gateway proxy endpoint.

        Args:
          name: Specify the name of the proxy endpoint.

          kind: The proxy endpoint kind

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        kind: Literal["identity"],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Create a new Zero Trust Gateway proxy endpoint.

        Args:
          kind: The proxy endpoint kind

          name: Specify the name of the proxy endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "name"], ["account_id", "kind", "name"])
    def create(
        self,
        *,
        account_id: str,
        name: str,
        kind: Literal["ip"] | Literal["identity"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            self._post(
                path_template("/accounts/{account_id}/gateway/proxy_endpoints", account_id=account_id),
                body=maybe_transform(
                    {
                        "name": name,
                        "kind": kind,
                    },
                    proxy_endpoint_create_params.ProxyEndpointCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ProxyEndpoint]:
        """
        List all Zero Trust Gateway proxy endpoints for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/proxy_endpoints", account_id=account_id),
            page=SyncSinglePage[ProxyEndpoint],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, ProxyEndpoint),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a configured Zero Trust Gateway proxy endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                account_id=account_id,
                proxy_endpoint_id=proxy_endpoint_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def edit(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        ips: SequenceNotStr[GatewayIPs] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Update a configured Zero Trust Gateway proxy endpoint.

        Args:
          ips: Specify the list of CIDRs to restrict ingress connections.

          name: Specify the name of the proxy endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            self._patch(
                path_template(
                    "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                    account_id=account_id,
                    proxy_endpoint_id=proxy_endpoint_id,
                ),
                body=maybe_transform(
                    {
                        "ips": ips,
                        "name": name,
                    },
                    proxy_endpoint_edit_params.ProxyEndpointEditParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Get a single Zero Trust Gateway proxy endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            self._get(
                path_template(
                    "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                    account_id=account_id,
                    proxy_endpoint_id=proxy_endpoint_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncProxyEndpointsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProxyEndpointsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncProxyEndpointsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProxyEndpointsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncProxyEndpointsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        name: str,
        kind: Literal["ip"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Create a new Zero Trust Gateway proxy endpoint.

        Args:
          name: Specify the name of the proxy endpoint.

          kind: The proxy endpoint kind

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        kind: Literal["identity"],
        name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Create a new Zero Trust Gateway proxy endpoint.

        Args:
          kind: The proxy endpoint kind

          name: Specify the name of the proxy endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "name"], ["account_id", "kind", "name"])
    async def create(
        self,
        *,
        account_id: str,
        name: str,
        kind: Literal["ip"] | Literal["identity"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            await self._post(
                path_template("/accounts/{account_id}/gateway/proxy_endpoints", account_id=account_id),
                body=await async_maybe_transform(
                    {
                        "name": name,
                        "kind": kind,
                    },
                    proxy_endpoint_create_params.ProxyEndpointCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProxyEndpoint, AsyncSinglePage[ProxyEndpoint]]:
        """
        List all Zero Trust Gateway proxy endpoints for an account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/gateway/proxy_endpoints", account_id=account_id),
            page=AsyncSinglePage[ProxyEndpoint],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=cast(Any, ProxyEndpoint),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a configured Zero Trust Gateway proxy endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                account_id=account_id,
                proxy_endpoint_id=proxy_endpoint_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def edit(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        ips: SequenceNotStr[GatewayIPs] | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Update a configured Zero Trust Gateway proxy endpoint.

        Args:
          ips: Specify the list of CIDRs to restrict ingress connections.

          name: Specify the name of the proxy endpoint.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            await self._patch(
                path_template(
                    "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                    account_id=account_id,
                    proxy_endpoint_id=proxy_endpoint_id,
                ),
                body=await async_maybe_transform(
                    {
                        "ips": ips,
                        "name": name,
                    },
                    proxy_endpoint_edit_params.ProxyEndpointEditParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get(
        self,
        proxy_endpoint_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ProxyEndpoint]:
        """
        Get a single Zero Trust Gateway proxy endpoint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not proxy_endpoint_id:
            raise ValueError(f"Expected a non-empty value for `proxy_endpoint_id` but received {proxy_endpoint_id!r}")
        return cast(
            Optional[ProxyEndpoint],
            await self._get(
                path_template(
                    "/accounts/{account_id}/gateway/proxy_endpoints/{proxy_endpoint_id}",
                    account_id=account_id,
                    proxy_endpoint_id=proxy_endpoint_id,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[ProxyEndpoint]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[ProxyEndpoint]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class ProxyEndpointsResourceWithRawResponse:
    def __init__(self, proxy_endpoints: ProxyEndpointsResource) -> None:
        self._proxy_endpoints = proxy_endpoints

        self.create = to_raw_response_wrapper(
            proxy_endpoints.create,
        )
        self.list = to_raw_response_wrapper(
            proxy_endpoints.list,
        )
        self.delete = to_raw_response_wrapper(
            proxy_endpoints.delete,
        )
        self.edit = to_raw_response_wrapper(
            proxy_endpoints.edit,
        )
        self.get = to_raw_response_wrapper(
            proxy_endpoints.get,
        )


class AsyncProxyEndpointsResourceWithRawResponse:
    def __init__(self, proxy_endpoints: AsyncProxyEndpointsResource) -> None:
        self._proxy_endpoints = proxy_endpoints

        self.create = async_to_raw_response_wrapper(
            proxy_endpoints.create,
        )
        self.list = async_to_raw_response_wrapper(
            proxy_endpoints.list,
        )
        self.delete = async_to_raw_response_wrapper(
            proxy_endpoints.delete,
        )
        self.edit = async_to_raw_response_wrapper(
            proxy_endpoints.edit,
        )
        self.get = async_to_raw_response_wrapper(
            proxy_endpoints.get,
        )


class ProxyEndpointsResourceWithStreamingResponse:
    def __init__(self, proxy_endpoints: ProxyEndpointsResource) -> None:
        self._proxy_endpoints = proxy_endpoints

        self.create = to_streamed_response_wrapper(
            proxy_endpoints.create,
        )
        self.list = to_streamed_response_wrapper(
            proxy_endpoints.list,
        )
        self.delete = to_streamed_response_wrapper(
            proxy_endpoints.delete,
        )
        self.edit = to_streamed_response_wrapper(
            proxy_endpoints.edit,
        )
        self.get = to_streamed_response_wrapper(
            proxy_endpoints.get,
        )


class AsyncProxyEndpointsResourceWithStreamingResponse:
    def __init__(self, proxy_endpoints: AsyncProxyEndpointsResource) -> None:
        self._proxy_endpoints = proxy_endpoints

        self.create = async_to_streamed_response_wrapper(
            proxy_endpoints.create,
        )
        self.list = async_to_streamed_response_wrapper(
            proxy_endpoints.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            proxy_endpoints.delete,
        )
        self.edit = async_to_streamed_response_wrapper(
            proxy_endpoints.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            proxy_endpoints.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TunnelParam"]


class TunnelParam(TypedDict, total=False):
    """Configuration for using Keyless SSL through a Cloudflare Tunnel"""

    private_ip: Required[str]
    """Private IP of the Key Server Host"""

    vnet_id: Required[str]
    """Cloudflare Tunnel Virtual Network ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

from ..._models import BaseModel

__all__ = ["TraceItem"]


class TraceItem(BaseModel):
    """List of steps acting on request/response"""

    action: Optional[str] = None
    """If step type is rule, then action performed by this rule"""

    action_parameters: Optional[object] = None
    """If step type is rule, then action parameters of this rule as JSON"""

    description: Optional[str] = None
    """If step type is rule or ruleset, the description of this entity"""

    expression: Optional[str] = None
    """If step type is rule, then expression used to match for this rule"""

    kind: Optional[str] = None
    """If step type is ruleset, then kind of this ruleset"""

    matched: Optional[bool] = None
    """Whether tracing step affected tracing request/response"""

    name: Optional[str] = None
    """If step type is ruleset, then name of this ruleset"""

    step_name: Optional[str] = None
    """Tracing step identifying name"""

    trace: Optional["Trace"] = None

    type: Optional[str] = None
    """Tracing step type"""


from .trace import Trace



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ......_types import Body, Query, Headers, NotGiven, not_given
from ......_utils import path_template, maybe_transform
from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from ......_response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ......pagination import SyncSinglePage, AsyncSinglePage
from ......_base_client import AsyncPaginator, make_request_options
from ......types.zero_trust.devices.fallback_domain import FallbackDomain
from ......types.zero_trust.devices.fallback_domain_param import FallbackDomainParam

__all__ = ["FallbackDomainsResource", "AsyncFallbackDomainsResource"]


class FallbackDomainsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FallbackDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FallbackDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FallbackDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FallbackDomainsResourceWithStreamingResponse(self)

    def update(
        self,
        policy_id: str,
        *,
        account_id: str,
        domains: Iterable[FallbackDomainParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[FallbackDomain]:
        """Sets the list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead. This will only apply to the
        specified device settings profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/policy/{policy_id}/fallback_domains",
                account_id=account_id,
                policy_id=policy_id,
            ),
            page=SyncSinglePage[FallbackDomain],
            body=maybe_transform(domains, Iterable[FallbackDomainParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
            method="put",
        )

    def get(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[FallbackDomain]:
        """
        Fetches the list of domains to bypass Gateway DNS resolution from a specified
        device settings profile. These domains will use the specified local DNS resolver
        instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/policy/{policy_id}/fallback_domains",
                account_id=account_id,
                policy_id=policy_id,
            ),
            page=SyncSinglePage[FallbackDomain],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
        )


class AsyncFallbackDomainsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFallbackDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFallbackDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFallbackDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFallbackDomainsResourceWithStreamingResponse(self)

    def update(
        self,
        policy_id: str,
        *,
        account_id: str,
        domains: Iterable[FallbackDomainParam],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FallbackDomain, AsyncSinglePage[FallbackDomain]]:
        """Sets the list of domains to bypass Gateway DNS resolution.

        These domains will
        use the specified local DNS resolver instead. This will only apply to the
        specified device settings profile.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/policy/{policy_id}/fallback_domains",
                account_id=account_id,
                policy_id=policy_id,
            ),
            page=AsyncSinglePage[FallbackDomain],
            body=maybe_transform(domains, Iterable[FallbackDomainParam]),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
            method="put",
        )

    def get(
        self,
        policy_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FallbackDomain, AsyncSinglePage[FallbackDomain]]:
        """
        Fetches the list of domains to bypass Gateway DNS resolution from a specified
        device settings profile. These domains will use the specified local DNS resolver
        instead.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not policy_id:
            raise ValueError(f"Expected a non-empty value for `policy_id` but received {policy_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/devices/policy/{policy_id}/fallback_domains",
                account_id=account_id,
                policy_id=policy_id,
            ),
            page=AsyncSinglePage[FallbackDomain],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=FallbackDomain,
        )


class FallbackDomainsResourceWithRawResponse:
    def __init__(self, fallback_domains: FallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = to_raw_response_wrapper(
            fallback_domains.update,
        )
        self.get = to_raw_response_wrapper(
            fallback_domains.get,
        )


class AsyncFallbackDomainsResourceWithRawResponse:
    def __init__(self, fallback_domains: AsyncFallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = async_to_raw_response_wrapper(
            fallback_domains.update,
        )
        self.get = async_to_raw_response_wrapper(
            fallback_domains.get,
        )


class FallbackDomainsResourceWithStreamingResponse:
    def __init__(self, fallback_domains: FallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = to_streamed_response_wrapper(
            fallback_domains.update,
        )
        self.get = to_streamed_response_wrapper(
            fallback_domains.get,
        )


class AsyncFallbackDomainsResourceWithStreamingResponse:
    def __init__(self, fallback_domains: AsyncFallbackDomainsResource) -> None:
        self._fallback_domains = fallback_domains

        self.update = async_to_streamed_response_wrapper(
            fallback_domains.update,
        )
        self.get = async_to_streamed_response_wrapper(
            fallback_domains.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["CertificateAuthority"]

CertificateAuthority: TypeAlias = Literal["google", "lets_encrypt", "ssl_com"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .verified_bots import (
    VerifiedBotsResource,
    AsyncVerifiedBotsResource,
    VerifiedBotsResourceWithRawResponse,
    AsyncVerifiedBotsResourceWithRawResponse,
    VerifiedBotsResourceWithStreamingResponse,
    AsyncVerifiedBotsResourceWithStreamingResponse,
)

__all__ = [
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
    "VerifiedBotsResource",
    "AsyncVerifiedBotsResource",
    "VerifiedBotsResourceWithRawResponse",
    "AsyncVerifiedBotsResourceWithRawResponse",
    "VerifiedBotsResourceWithStreamingResponse",
    "AsyncVerifiedBotsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel
from ..aggregate_time_period import AggregateTimePeriod
from ...digital_experience_monitor import DigitalExperienceMonitor

__all__ = [
    "Tests",
    "OverviewMetrics",
    "Test",
    "TestHTTPResults",
    "TestHTTPResultsResourceFetchTime",
    "TestHTTPResultsResourceFetchTimeHistory",
    "TestHTTPResultsResourceFetchTimeOverTime",
    "TestHTTPResultsResourceFetchTimeOverTimeValue",
    "TestHTTPResultsByColo",
    "TestHTTPResultsByColoResourceFetchTime",
    "TestHTTPResultsByColoResourceFetchTimeHistory",
    "TestHTTPResultsByColoResourceFetchTimeOverTime",
    "TestHTTPResultsByColoResourceFetchTimeOverTimeValue",
    "TestTracerouteResults",
    "TestTracerouteResultsRoundTripTime",
    "TestTracerouteResultsRoundTripTimeHistory",
    "TestTracerouteResultsRoundTripTimeOverTime",
    "TestTracerouteResultsRoundTripTimeOverTimeValue",
    "TestTracerouteResultsByColo",
    "TestTracerouteResultsByColoRoundTripTime",
    "TestTracerouteResultsByColoRoundTripTimeHistory",
    "TestTracerouteResultsByColoRoundTripTimeOverTime",
    "TestTracerouteResultsByColoRoundTripTimeOverTimeValue",
]


class OverviewMetrics(BaseModel):
    tests_total: int = FieldInfo(alias="testsTotal")
    """number of tests."""

    avg_http_availability_pct: Optional[float] = FieldInfo(alias="avgHttpAvailabilityPct", default=None)
    """percentage availability for all HTTP test results in response"""

    avg_traceroute_availability_pct: Optional[float] = FieldInfo(alias="avgTracerouteAvailabilityPct", default=None)
    """percentage availability for all traceroutes results in response"""


class TestHTTPResultsResourceFetchTimeHistory(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    delta_pct: Optional[float] = FieldInfo(alias="deltaPct", default=None)


class TestHTTPResultsResourceFetchTimeOverTimeValue(BaseModel):
    __test__ = False
    avg_ms: int = FieldInfo(alias="avgMs")

    timestamp: str


class TestHTTPResultsResourceFetchTimeOverTime(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    values: List[TestHTTPResultsResourceFetchTimeOverTimeValue]


class TestHTTPResultsResourceFetchTime(BaseModel):
    __test__ = False
    history: List[TestHTTPResultsResourceFetchTimeHistory]

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    over_time: Optional[TestHTTPResultsResourceFetchTimeOverTime] = FieldInfo(alias="overTime", default=None)


class TestHTTPResults(BaseModel):
    __test__ = False
    resource_fetch_time: TestHTTPResultsResourceFetchTime = FieldInfo(alias="resourceFetchTime")


class TestHTTPResultsByColoResourceFetchTimeHistory(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    delta_pct: Optional[float] = FieldInfo(alias="deltaPct", default=None)


class TestHTTPResultsByColoResourceFetchTimeOverTimeValue(BaseModel):
    __test__ = False
    avg_ms: int = FieldInfo(alias="avgMs")

    timestamp: str


class TestHTTPResultsByColoResourceFetchTimeOverTime(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    values: List[TestHTTPResultsByColoResourceFetchTimeOverTimeValue]


class TestHTTPResultsByColoResourceFetchTime(BaseModel):
    __test__ = False
    history: List[TestHTTPResultsByColoResourceFetchTimeHistory]

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    over_time: Optional[TestHTTPResultsByColoResourceFetchTimeOverTime] = FieldInfo(alias="overTime", default=None)


class TestHTTPResultsByColo(BaseModel):
    __test__ = False
    colo: str
    """Cloudflare colo"""

    resource_fetch_time: TestHTTPResultsByColoResourceFetchTime = FieldInfo(alias="resourceFetchTime")


class TestTracerouteResultsRoundTripTimeHistory(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    delta_pct: Optional[float] = FieldInfo(alias="deltaPct", default=None)


class TestTracerouteResultsRoundTripTimeOverTimeValue(BaseModel):
    __test__ = False
    avg_ms: int = FieldInfo(alias="avgMs")

    timestamp: str


class TestTracerouteResultsRoundTripTimeOverTime(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    values: List[TestTracerouteResultsRoundTripTimeOverTimeValue]


class TestTracerouteResultsRoundTripTime(BaseModel):
    __test__ = False
    history: List[TestTracerouteResultsRoundTripTimeHistory]

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    over_time: Optional[TestTracerouteResultsRoundTripTimeOverTime] = FieldInfo(alias="overTime", default=None)


class TestTracerouteResults(BaseModel):
    __test__ = False
    round_trip_time: TestTracerouteResultsRoundTripTime = FieldInfo(alias="roundTripTime")


class TestTracerouteResultsByColoRoundTripTimeHistory(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    delta_pct: Optional[float] = FieldInfo(alias="deltaPct", default=None)


class TestTracerouteResultsByColoRoundTripTimeOverTimeValue(BaseModel):
    __test__ = False
    avg_ms: int = FieldInfo(alias="avgMs")

    timestamp: str


class TestTracerouteResultsByColoRoundTripTimeOverTime(BaseModel):
    __test__ = False
    time_period: AggregateTimePeriod = FieldInfo(alias="timePeriod")

    values: List[TestTracerouteResultsByColoRoundTripTimeOverTimeValue]


class TestTracerouteResultsByColoRoundTripTime(BaseModel):
    __test__ = False
    history: List[TestTracerouteResultsByColoRoundTripTimeHistory]

    avg_ms: Optional[int] = FieldInfo(alias="avgMs", default=None)

    over_time: Optional[TestTracerouteResultsByColoRoundTripTimeOverTime] = FieldInfo(alias="overTime", default=None)


class TestTracerouteResultsByColo(BaseModel):
    __test__ = False
    colo: str
    """Cloudflare colo"""

    round_trip_time: TestTracerouteResultsByColoRoundTripTime = FieldInfo(alias="roundTripTime")


class Test(BaseModel):
    __test__ = False
    id: str
    """API Resource UUID tag."""

    created: str
    """date the test was created."""

    description: str
    """the test description defined during configuration"""

    enabled: bool
    """if true, then the test will run on targeted devices.

    Else, the test will not run.
    """

    host: str

    interval: str
    """The interval at which the synthetic application test is set to run."""

    kind: Literal["http", "traceroute"]
    """test type, http or traceroute"""

    name: str
    """name given to this test"""

    updated: str

    http_results: Optional[TestHTTPResults] = FieldInfo(alias="httpResults", default=None)

    http_results_by_colo: Optional[List[TestHTTPResultsByColo]] = FieldInfo(alias="httpResultsByColo", default=None)

    method: Optional[str] = None
    """for HTTP, the method to use when running the test"""

    target_policies: Optional[List[DigitalExperienceMonitor]] = None

    targeted: Optional[bool] = None

    traceroute_results: Optional[TestTracerouteResults] = FieldInfo(alias="tracerouteResults", default=None)

    traceroute_results_by_colo: Optional[List[TestTracerouteResultsByColo]] = FieldInfo(
        alias="tracerouteResultsByColo", default=None
    )


class Tests(BaseModel):
    __test__ = False
    overview_metrics: OverviewMetrics = FieldInfo(alias="overviewMetrics")

    tests: List[Test]
    """array of test results objects."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["BrowserCheckParam"]


class BrowserCheckParam(TypedDict, total=False):
    id: Literal["browser_check"]
    """
    Inspect the visitor's browser for headers commonly associated with spammers and
    certain bots.
    """

    value: Literal["on", "off"]
    """The status of Browser Integrity Check."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DownloadGetResponse", "Audio", "Default"]


class Audio(BaseModel):
    """The audio-only download. Only present if this download type has been created."""

    percent_complete: float = FieldInfo(alias="percentComplete")
    """Indicates the progress as a percentage between 0 and 100."""

    status: Literal["ready", "inprogress", "error"]
    """The status of a generated download."""

    url: Optional[str] = None
    """The URL to access the generated download."""


class Default(BaseModel):
    """The default video download.

    Only present if this download type has been created.
    """

    percent_complete: float = FieldInfo(alias="percentComplete")
    """Indicates the progress as a percentage between 0 and 100."""

    status: Literal["ready", "inprogress", "error"]
    """The status of a generated download."""

    url: Optional[str] = None
    """The URL to access the generated download."""


class DownloadGetResponse(BaseModel):
    """An object with download type keys.

    Each key is optional and only present if that download type has been created.
    """

    audio: Optional[Audio] = None
    """The audio-only download. Only present if this download type has been created."""

    default: Optional[Default] = None
    """The default video download.

    Only present if this download type has been created.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["FleetStatusGetParams"]


class FleetStatusGetParams(TypedDict, total=False):
    account_id: Required[str]

    since_minutes: Required[float]
    """Number of minutes before current time"""

    colo: str
    """List of data centers to filter results"""

    time_now: str
    """Number of minutes before current time"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ......_utils import PropertyInfo
from ......_models import BaseModel

__all__ = [
    "BindingGetResponse",
    "WorkersBindingKindAI",
    "WorkersBindingKindAISearch",
    "WorkersBindingKindAISearchNamespace",
    "WorkersBindingKindAnalyticsEngine",
    "WorkersBindingKindAssets",
    "WorkersBindingKindBrowser",
    "WorkersBindingKindD1",
    "WorkersBindingKindDataBlob",
    "WorkersBindingKindDispatchNamespace",
    "WorkersBindingKindDispatchNamespaceOutbound",
    "WorkersBindingKindDispatchNamespaceOutboundParam",
    "WorkersBindingKindDispatchNamespaceOutboundWorker",
    "WorkersBindingKindDurableObjectNamespace",
    "WorkersBindingKindHyperdrive",
    "WorkersBindingKindInherit",
    "WorkersBindingKindImages",
    "WorkersBindingKindJson",
    "WorkersBindingKindKVNamespace",
    "WorkersBindingKindMedia",
    "WorkersBindingKindMTLSCertificate",
    "WorkersBindingKindPlainText",
    "WorkersBindingKindPipelines",
    "WorkersBindingKindQueue",
    "WorkersBindingKindRatelimit",
    "WorkersBindingKindRatelimitSimple",
    "WorkersBindingKindR2Bucket",
    "WorkersBindingKindSecretText",
    "WorkersBindingKindSendEmail",
    "WorkersBindingKindService",
    "WorkersBindingKindTextBlob",
    "WorkersBindingKindVectorize",
    "WorkersBindingKindVersionMetadata",
    "WorkersBindingKindSecretsStoreSecret",
    "WorkersBindingKindFlagship",
    "WorkersBindingKindSecretKey",
    "WorkersBindingKindWorkflow",
    "WorkersBindingKindWasmModule",
    "WorkersBindingKindVPCService",
    "WorkersBindingKindVPCNetwork",
]


class WorkersBindingKindAI(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindAISearch(BaseModel):
    instance_name: str
    """The user-chosen instance name.

    Must exist at deploy time. The worker can search, chat, update, and manage
    items/jobs on this instance.
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["ai_search"]
    """The kind of resource that the binding provides."""

    namespace: Optional[str] = None
    """The namespace the instance belongs to.

    Defaults to "default" if omitted. Customers who don't use namespaces can simply
    omit this field.
    """


class WorkersBindingKindAISearchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The user-chosen namespace name.

    Must exist before deploy -- Wrangler handles auto-creation on deploy failure (R2
    bucket pattern). The "default" namespace is auto-created by config-api for new
    accounts. Grants full access (CRUD + search + chat) to all instances within the
    namespace.
    """

    type: Literal["ai_search_namespace"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindAnalyticsEngine(BaseModel):
    dataset: str
    """The name of the dataset to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["analytics_engine"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindAssets(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["assets"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindBrowser(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["browser"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindD1(BaseModel):
    database_id: str
    """Identifier of the D1 database to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["d1"]
    """The kind of resource that the binding provides."""

    id: Optional[str] = None
    """Identifier of the D1 database to bind to."""


class WorkersBindingKindDataBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the data content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["data_blob"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindDispatchNamespaceOutboundParam(BaseModel):
    name: str
    """Name of the parameter."""


class WorkersBindingKindDispatchNamespaceOutboundWorker(BaseModel):
    """Outbound worker."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the outbound worker."""

    environment: Optional[str] = None
    """Environment of the outbound worker."""

    service: Optional[str] = None
    """Name of the outbound worker."""


class WorkersBindingKindDispatchNamespaceOutbound(BaseModel):
    """Outbound worker."""

    params: Optional[List[WorkersBindingKindDispatchNamespaceOutboundParam]] = None
    """
    Pass information from the Dispatch Worker to the Outbound Worker through the
    parameters.
    """

    worker: Optional[WorkersBindingKindDispatchNamespaceOutboundWorker] = None
    """Outbound worker."""


class WorkersBindingKindDispatchNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace: str
    """The name of the dispatch namespace."""

    type: Literal["dispatch_namespace"]
    """The kind of resource that the binding provides."""

    outbound: Optional[WorkersBindingKindDispatchNamespaceOutbound] = None
    """Outbound worker."""


class WorkersBindingKindDurableObjectNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["durable_object_namespace"]
    """The kind of resource that the binding provides."""

    class_name: Optional[str] = None
    """The exported class name of the Durable Object."""

    dispatch_namespace: Optional[str] = None
    """The dispatch namespace the Durable Object script belongs to."""

    environment: Optional[str] = None
    """The environment of the script_name to bind to."""

    namespace_id: Optional[str] = None
    """Namespace identifier tag."""

    script_name: Optional[str] = None
    """
    The script where the Durable Object is defined, if it is external to this
    Worker.
    """


class WorkersBindingKindHyperdrive(BaseModel):
    id: str
    """Identifier of the Hyperdrive connection to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["hyperdrive"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindInherit(BaseModel):
    name: str
    """The name of the inherited binding."""

    type: Literal["inherit"]
    """The kind of resource that the binding provides."""

    old_name: Optional[str] = None
    """The old name of the inherited binding.

    If set, the binding will be renamed from `old_name` to `name` in the new
    version. If not set, the binding will keep the same name between versions.
    """

    version_id: Optional[str] = None
    """
    Identifier for the version to inherit the binding from, which can be the version
    ID or the literal "latest" to inherit from the latest version. Defaults to
    inheriting the binding from the latest version.
    """


class WorkersBindingKindImages(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["images"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindJson(BaseModel):
    json_: object = FieldInfo(alias="json")
    """JSON data to use."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["json"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindKVNamespace(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Namespace identifier tag."""

    type: Literal["kv_namespace"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindMedia(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["media"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindMTLSCertificate(BaseModel):
    certificate_id: str
    """Identifier of the certificate to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["mtls_certificate"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindPlainText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    text: str
    """The text value to use."""

    type: Literal["plain_text"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindPipelines(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    pipeline: str
    """Name of the Pipeline to bind to."""

    type: Literal["pipelines"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindQueue(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    queue_name: str
    """Name of the Queue to bind to."""

    type: Literal["queue"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindRatelimitSimple(BaseModel):
    """The rate limit configuration."""

    limit: float
    """The limit (requests per period)."""

    period: int
    """The period in seconds."""

    mitigation_timeout: Optional[int] = None
    """
    Duration in seconds to apply the mitigation action after the rate limit is
    exceeded. Valid values are 0 (disabled), 10, or multiples of 60 up to 86400.
    Must be greater than or equal to the period when non-zero.
    """


class WorkersBindingKindRatelimit(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    namespace_id: str
    """Identifier of the rate limit namespace to bind to."""

    simple: WorkersBindingKindRatelimitSimple
    """The rate limit configuration."""

    type: Literal["ratelimit"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindR2Bucket(BaseModel):
    bucket_name: str
    """R2 bucket to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["r2_bucket"]
    """The kind of resource that the binding provides."""

    jurisdiction: Optional[Literal["eu", "fedramp", "fedramp-high"]] = None
    """
    The
    [jurisdiction](https://developers.cloudflare.com/r2/reference/data-location/#jurisdictional-restrictions)
    of the R2 bucket.
    """


class WorkersBindingKindSecretText(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_text"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindSendEmail(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["send_email"]
    """The kind of resource that the binding provides."""

    allowed_destination_addresses: Optional[List[str]] = None
    """List of allowed destination addresses."""

    allowed_sender_addresses: Optional[List[str]] = None
    """List of allowed sender addresses."""

    destination_address: Optional[str] = None
    """Destination address for the email."""


class WorkersBindingKindService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service: str
    """Name of Worker to bind to."""

    type: Literal["service"]
    """The kind of resource that the binding provides."""

    entrypoint: Optional[str] = None
    """Entrypoint to invoke on the target Worker."""

    environment: Optional[str] = None
    """Optional environment if the Worker utilizes one."""


class WorkersBindingKindTextBlob(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the text content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["text_blob"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindVectorize(BaseModel):
    index_name: str
    """Name of the Vectorize index to bind to."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vectorize"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindVersionMetadata(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["version_metadata"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindSecretsStoreSecret(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    secret_name: str
    """Name of the secret in the store."""

    store_id: str
    """ID of the store containing the secret."""

    type: Literal["secrets_store_secret"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindFlagship(BaseModel):
    app_id: str
    """ID of the Flagship app to bind to for feature flag evaluation."""

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["flagship"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindSecretKey(BaseModel):
    algorithm: object
    """Algorithm-specific key parameters.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#algorithm).
    """

    format: Literal["raw", "pkcs8", "spki", "jwk"]
    """Data format of the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#format).
    """

    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["secret_key"]
    """The kind of resource that the binding provides."""

    usages: List[Literal["encrypt", "decrypt", "sign", "verify", "deriveKey", "deriveBits", "wrapKey", "unwrapKey"]]
    """Allowed operations with the key.

    [Learn more](https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey#keyUsages).
    """


class WorkersBindingKindWorkflow(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["workflow"]
    """The kind of resource that the binding provides."""

    workflow_name: str
    """Name of the Workflow to bind to."""

    class_name: Optional[str] = None
    """Class name of the Workflow.

    Should only be provided if the Workflow belongs to this script.
    """

    script_name: Optional[str] = None
    """Script name that contains the Workflow.

    If not provided, defaults to this script name.
    """


class WorkersBindingKindWasmModule(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    part: str
    """The name of the file containing the WebAssembly module content.

    Only accepted for `service worker syntax` Workers.
    """

    type: Literal["wasm_module"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindVPCService(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    service_id: str
    """Identifier of the VPC service to bind to."""

    type: Literal["vpc_service"]
    """The kind of resource that the binding provides."""


class WorkersBindingKindVPCNetwork(BaseModel):
    name: str
    """A JavaScript variable name for the binding."""

    type: Literal["vpc_network"]
    """The kind of resource that the binding provides."""

    network_id: Optional[str] = None
    """Identifier of the network to bind to.

    Only "cf1:network" is currently supported. Mutually exclusive with tunnel_id.
    """

    tunnel_id: Optional[str] = None
    """UUID of the Cloudflare Tunnel to bind to. Mutually exclusive with network_id."""


BindingGetResponse: TypeAlias = Annotated[
    Union[
        WorkersBindingKindAI,
        WorkersBindingKindAISearch,
        WorkersBindingKindAISearchNamespace,
        WorkersBindingKindAnalyticsEngine,
        WorkersBindingKindAssets,
        WorkersBindingKindBrowser,
        WorkersBindingKindD1,
        WorkersBindingKindDataBlob,
        WorkersBindingKindDispatchNamespace,
        WorkersBindingKindDurableObjectNamespace,
        WorkersBindingKindHyperdrive,
        WorkersBindingKindInherit,
        WorkersBindingKindImages,
        WorkersBindingKindJson,
        WorkersBindingKindKVNamespace,
        WorkersBindingKindMedia,
        WorkersBindingKindMTLSCertificate,
        WorkersBindingKindPlainText,
        WorkersBindingKindPipelines,
        WorkersBindingKindQueue,
        WorkersBindingKindRatelimit,
        WorkersBindingKindR2Bucket,
        WorkersBindingKindSecretText,
        WorkersBindingKindSendEmail,
        WorkersBindingKindService,
        WorkersBindingKindTextBlob,
        WorkersBindingKindVectorize,
        WorkersBindingKindVersionMetadata,
        WorkersBindingKindSecretsStoreSecret,
        WorkersBindingKindFlagship,
        WorkersBindingKindSecretKey,
        WorkersBindingKindWorkflow,
        WorkersBindingKindWasmModule,
        WorkersBindingKindVPCService,
        WorkersBindingKindVPCNetwork,
    ],
    PropertyInfo(discriminator="type"),
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["LabelListParams"]


class LabelListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    direction: Literal["asc", "desc"]
    """Direction to order results."""

    filter: str
    """Filter for labels where the name or description matches using substring match"""

    order: Literal["name", "description", "created_at", "last_updated", "mapped_resources.operations"]
    """Field to order by"""

    page: int
    """Page number of paginated results."""

    per_page: int
    """Maximum number of results per page."""

    source: Literal["user", "managed"]
    """Filter for labels with source"""

    with_mapped_resource_counts: bool
    """Include `mapped_resources` for each label"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ThreatEventEditParams", "Raw"]


class ThreatEventEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    dataset_id: Required[Annotated[str, PropertyInfo(alias="datasetId")]]
    """Dataset ID containing the event to update."""

    attacker: Optional[str]

    attacker_country: Annotated[str, PropertyInfo(alias="attackerCountry")]

    category: str

    created_at: Annotated[Union[str, datetime], PropertyInfo(alias="createdAt", format="iso8601")]

    date: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    event: str

    indicator: str

    indicator_type: Annotated[str, PropertyInfo(alias="indicatorType")]

    insight: str

    raw: Raw

    target_country: Annotated[str, PropertyInfo(alias="targetCountry")]

    target_industry: Annotated[str, PropertyInfo(alias="targetIndustry")]

    tlp: str


class Raw(TypedDict, total=False):
    data: Optional[Dict[str, object]]

    source: str

    tlp: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .mtls_certificate import MTLSCertificate as MTLSCertificate
from .certificate_asssociation import CertificateAsssociation as CertificateAsssociation
from .mtls_certificate_list_params import MTLSCertificateListParams as MTLSCertificateListParams
from .mtls_certificate_create_params import MTLSCertificateCreateParams as MTLSCertificateCreateParams
from .mtls_certificate_create_response import MTLSCertificateCreateResponse as MTLSCertificateCreateResponse
